/*

    <Linux PAM module for Dallas Semiconductor's iButtons - v2.1alfa>
    <Jose Rufino - rufino@ipb.pt, September 4th 1999                >
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
      
*/


#define PAM_SM_AUTH
#define PAM_SM_PASSWORD

#include <security/pam_modules.h>

#include <syslog.h>
#include <pwd.h>
#include <sys/types.h>
#include <string.h>

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>

#include <mlan.h>

#include "md5.h"

/*
#include <unistd.h>
*/
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <shadow.h>

int iButton_get_local_IP_addr(char *, int);

/*----------------------------------------------------------------------------*/

PAM_EXTERN int
pam_sm_authenticate (pam_handle_t * pamh, int flags, int argc, const char **argv)
{
  int            ibutton_debug=FALSE;
  const void     *item;
  char           pam_user[32];
  char           localIPaddress[32];
  char           mlan_return_msg[128];
  int            retval, i, j;
  uchar          ibutton_rom[8];
  uchar          ibutton_content[29];
  struct spwd    *sp;
  int            ibutton_match;

  if (argc>=1 && !strcmp(argv[0], "debug")) {
    openlog("pam_ibutton", LOG_PERROR, LOG_AUTHPRIV);
    ibutton_debug=TRUE;
  }

  /* collect user from previous module */
  if (pam_get_item(pamh, PAM_USER, &item) != PAM_SUCCESS) {
    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_ERR, "couldn't get user from previous module");
    return(PAM_USER_UNKNOWN);          
  }

  /* save user from previous module */
  strncpy(pam_user, item, 32);
  pam_user[31]='\0';
/*
  if (ibutton_debug)
    syslog(LOG_AUTHPRIV | LOG_DEBUG, "user from previous module is <%s>", pam_user); 
*/

  /* if user is not "ibutton" then this module is not necessary */
  /* and so we need to ensure that pam_pwdb executes: we return an error ... */
  if (strcmp(pam_user, "ibutton")) {
/*
    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_ERR, "user is not <ibutton>; iButton authentication will not take place");
*/
    return(PAM_USER_UNKNOWN);
  }

/*
  if (ibutton_debug)
    syslog(LOG_AUTHPRIV | LOG_DEBUG, "user is <ibutton>; iButton authentication will take place");
*/

  /* get my IP address ... */
  if (iButton_get_local_IP_addr(localIPaddress, 32)<0) {
    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_ERR, "couldn't get my IP address");
    return(PAM_AUTH_ERR);
  }

  // attempt to aquire the 1-Wire Net
  if (!Aquire1WireNet("/dev/iButton", mlan_return_msg)) {
    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_ERR, "%s", mlan_return_msg);
    return(PAM_AUTH_ERR);
  }
/*
  if (ibutton_debug)
    syslog(LOG_AUTHPRIV | LOG_DEBUG, "%s", mlan_return_msg);
*/

  // find all iButton devices on the 1-Wire Net and select one
  {
       uchar ibutton_roms[8][8]; /* pool of ROMS */
       int   j,k;

       printf("Please select the iButton to read.\n");
       printf("-------------------- Start of search\n");
       i=0;
       // find the first device (all devices not just alarming)
       retval = MLanFirst(TRUE, FALSE);
       while (i!= 8 && retval) {
         // print the device number
         printf("(%d) ",++i);   

         // save & print the Serial Number of the device just found
         MLanSerialNum(ibutton_roms[i-1],TRUE);
         for (j = 7; j >= 0; j--)
           printf("%02X",ibutton_roms[i-1][j]);
         printf("\n");

         // find the next device if pool is not yet full
         if (i!=8)
           retval = MLanNext(TRUE, FALSE);
       }
       printf("-------------------- End of search\n");            

       /* if none iButton found, exit ... */
       if (i==0) {
         if (ibutton_debug)
           syslog(LOG_AUTHPRIV | LOG_ERR, "couldn't found any iButton");
         Release1WireNet(mlan_return_msg); // release the 1-Wire Net
         if (ibutton_debug)
           syslog(LOG_AUTHPRIV | LOG_DEBUG, "%s", mlan_return_msg);
         return(PAM_AUTH_ERR);
       }

       /* save ROM of the selected iButton */
       printf("(<ENTER> to select first device, <number> to select specific device) ");
       k=getchar();
       if (k!=10 && (k<'1' || (k-'1'+1)>i)) {
         if (ibutton_debug)
           syslog(LOG_AUTHPRIV | LOG_ERR, "invalid selection");
         Release1WireNet(mlan_return_msg); // release the 1-Wire Net
         return (PAM_AUTH_ERR);
       }
       k=(k==10? 1 : k-'1'+1);
       bcopy(ibutton_roms[k-1], ibutton_rom, 8);
  }

  /* search our iButton */
  MLanSerialNum(ibutton_rom, FALSE);
  if (!MLanAccess()) {
    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_ERR, "couldn't find the iButton on the bus");
    Release1WireNet(mlan_return_msg); // release the 1-Wire Net
    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_DEBUG, "%s", mlan_return_msg);
    return (PAM_AUTH_ERR);
  }                     

  /* read content */
  if (MLanReadPacketStd(1, 0, ibutton_content)<=0) {
    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_ERR, "couldn't read the iButton");
    Release1WireNet(mlan_return_msg); // release the 1-Wire Net
    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_DEBUG, "%s", mlan_return_msg);
    return (PAM_AUTH_ERR);
  }      

  /* iButton access is not needed anymore */
  Release1WireNet(mlan_return_msg); // release the 1-Wire Net
/*
  if (ibutton_debug)
    syslog(LOG_AUTHPRIV | LOG_DEBUG, "%s", mlan_return_msg);
*/

  /* try to match iButton content with MD5 (IP, ROM, /etc/shadow entry) */
  {
    uchar       dummy_content[256];
    uchar       *c;

    MD5_CTX     md5_ctx;
    uchar       md5_digest[16]; 

    /* begin search */
    setspent();

    /* exaustive search on /etc/shadow */
    ibutton_match=FALSE;
    while(sp=getspent()) {

      /* assemble content before aplying MD5 */
      bzero(dummy_content, 256);
      c=dummy_content;

      /* add local IP address */
      strcpy(c, localIPaddress);
      c+=strlen(localIPaddress)+1;

      /* add iButton serial number */
      bcopy(ibutton_rom, c, 8);
      c+=8;

      /* add /etc/shadow entry */
      sprintf(c, "%s:%s:%ld:%d:%d:%d:%d:%d:%d", sp->sp_namp, sp->sp_pwdp, sp->sp_lstchg, sp->sp_min, sp->sp_max, sp->sp_warn, sp->sp_inact, sp->sp_expire, sp->sp_flag);
      c+=strlen(c)+1;

      /* apply MD5 */
      MD5Init(&md5_ctx);
      MD5Update(&md5_ctx,dummy_content,c-dummy_content);
      MD5Final(md5_digest, &md5_ctx);       

      /* try to match local digest with iButton's digest */
/*
      if (ibutton_debug)
        syslog(LOG_AUTHPRIV | LOG_DEBUG, "trying to match with <%s>", sp->sp_namp);
*/
      if(!bcmp(md5_digest, ibutton_content, 16)) {
        ibutton_match=TRUE;
        break;
      }
    }

    /* end search */
    endspent();
  }

  /* no match ... */
  if (ibutton_match==FALSE) {
    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_DEBUG, "no match");
    /* set username to nobody */
    pam_set_item(pamh, PAM_USER, "nobody");
    return (PAM_AUTH_ERR);
  }

/*
  if (ibutton_debug)
    syslog(LOG_AUTHPRIV | LOG_DEBUG, "match with <%s>", sp->sp_namp);
*/

  /* set username to the matching one */
  pam_set_item(pamh, PAM_USER, sp->sp_namp);

  return(PAM_SUCCESS);
}

/*----------------------------------------------------------------------------*/

PAM_EXTERN int
pam_sm_setcred (pam_handle_t * pamh, int flags, int argc, const char **argv)
{
  return PAM_SUCCESS;
}

/*----------------------------------------------------------------------------*/

PAM_EXTERN int
pam_sm_chauthtok (pam_handle_t * pamh, int flags, int argc, const char **argv)
{
  int            ibutton_debug=FALSE;
  static char    pam_user[32];
  static char    localIPaddress[32];
  char           mlan_return_msg[128];
  static uchar   ibutton_rom[8];


  if (argc>=1 && !strcmp(argv[0], "debug")) {
    openlog("pam_ibutton", LOG_PERROR, LOG_AUTHPRIV);
    ibutton_debug=TRUE;
  }

  /* NOTE:
     The Linux-PAM Module Writer's Guide", at page 13, states that
     ...the Linux-PAM library calls this function twice in sucession ...
     ...it is only on the 2nd call that the authorization token is changed ...
  */

  /* 1st time: setup everything possible, preparing for the 2nd phase  */
  if (flags & PAM_PRELIM_CHECK) {
    const void     *item;
    struct passwd  *pw;
    int            retval, i;


    /* collect user from previous module */
    if (pam_get_item(pamh, PAM_USER, &item) != PAM_SUCCESS) {
      if (ibutton_debug)
        syslog(LOG_AUTHPRIV | LOG_ERR, "couldn't get user from previous module");
      return(PAM_USER_UNKNOWN);
    }

    /* save user from previous module */
    strncpy(pam_user, item, 32);
    pam_user[31]='\0';
/*
    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_DEBUG, "user from previous module is <%s>", pam_user);
*/

    /* who am I, really ? if unknown, exit ... */
    if ((pw=getpwuid(getuid()))==NULL) {
      if (ibutton_debug)
        syslog(LOG_AUTHPRIV | LOG_ERR, "couldn't know who am I");
      return(PAM_USER_UNKNOWN);
    }

    /* if unauthorized user, exit ... */
    if (strcmp("root", pw->pw_name) && strcmp(pam_user, pw->pw_name)) {
      if (ibutton_debug)
        syslog(LOG_AUTHPRIV | LOG_ERR, "you are not <root> nor <%s>", pam_user);
      return(PAM_PERM_DENIED);
    }

    /* get my IP address ... */
    if (iButton_get_local_IP_addr(localIPaddress, 32)<0) {
      if (ibutton_debug)
        syslog(LOG_AUTHPRIV | LOG_ERR, "couldn't get my IP address");
      return(PAM_PERM_DENIED);
    }

    // attempt to aquire the 1-Wire Net
    if (!Aquire1WireNet("/dev/iButton", mlan_return_msg)) {
      if (ibutton_debug)
        syslog(LOG_AUTHPRIV | LOG_ERR, "%s", mlan_return_msg);
      return(PAM_PERM_DENIED);
    }
/*
    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_DEBUG, "%s", mlan_return_msg);
*/
   
    // find all iButton devices on the 1-Wire Net and select one
    {
        uchar ibutton_roms[8][8]; /* pool of ROMS */
        int   j,k;

        printf("Please select the iButton to write.\n");
        printf("-------------------- Start of search\n");
        i=0;
        // find the first device (all devices not just alarming)
        retval = MLanFirst(TRUE, FALSE);
        while (i!= 8 && retval) {
          // print the device number
          printf("(%d) ",++i);

          // save & print the Serial Number of the device just found
          MLanSerialNum(ibutton_roms[i-1],TRUE);
          for (j = 7; j >= 0; j--)
            printf("%02X",ibutton_roms[i-1][j]);
          printf("\n");                          

          // find the next device if pool is not yet full
          if (i!=8)
            retval = MLanNext(TRUE, FALSE);
        }
        printf("-------------------- End of search\n");

        /* if none iButton found, exit ... */
        if (i==0) {
          if (ibutton_debug)
            syslog(LOG_AUTHPRIV | LOG_ERR, "couldn't found any iButton");
          Release1WireNet(mlan_return_msg); // release the 1-Wire Net
          if (ibutton_debug)
            syslog(LOG_AUTHPRIV | LOG_DEBUG, "%s", mlan_return_msg);
          return(PAM_PERM_DENIED);
        }

        /* save ROM of the selected iButton */
        printf("(<ENTER> to select first device, <number> to select specific device) ");
        k=getchar();
        if (k!=10 && (k<'1' || (k-'1'+1)>i)) {
          if (ibutton_debug)
            syslog(LOG_AUTHPRIV | LOG_ERR, "invalid selection");
          Release1WireNet(mlan_return_msg); // release the 1-Wire Net
          return (PAM_PERM_DENIED);
        }
        k=(k==10? 1 : k-'1'+1);
        bcopy(ibutton_roms[k-1], ibutton_rom, 8);
    }

    Release1WireNet(mlan_return_msg); // release the 1-Wire Net
/*
    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_DEBUG, "%s", mlan_return_msg);
*/
    return (PAM_SUCCESS);

  }

  /* 2nd time: try to update iButton */
  if (flags & PAM_UPDATE_AUTHTOK) {
    struct spwd *sp;
    uchar       ibutton_content[256];
    uchar       *c;

    MD5_CTX     md5_ctx;
    uchar       md5_digest[16];
 
    /* now that /etc/shadow has been updated, get pam_user entry */
    if ((sp=getspnam(pam_user))==NULL) {
      if (ibutton_debug)
        syslog(LOG_AUTHPRIV | LOG_DEBUG, "couldn't read /etc/shadow entry for <%s>", pam_user);
      return (PAM_AUTHINFO_UNAVAIL);
    }

    /* assemble content before aplying MD5 */
    bzero(ibutton_content, 256);
    c=ibutton_content;

    /* add local IP address */
    strcpy(c, localIPaddress);
    c+=strlen(localIPaddress)+1;

    /* add iButton serial number */
    bcopy(ibutton_rom, c, 8);
    c+=8;

    /* add /etc/shadow entry */
    sprintf(c, "%s:%s:%ld:%d:%d:%d:%d:%d:%d", sp->sp_namp, sp->sp_pwdp, sp->sp_lstchg, sp->sp_min, sp->sp_max, sp->sp_warn, sp->sp_inact, sp->sp_expire, sp->sp_flag);
    c+=strlen(c)+1;

    /* apply MD5 */
    MD5Init(&md5_ctx);
    MD5Update(&md5_ctx,ibutton_content,c-ibutton_content);
    MD5Final(md5_digest, &md5_ctx);

    /* update the previously selected (on 1st phase) iButton */

    // attempt to aquire the 1-Wire Net
    if (!Aquire1WireNet("/dev/iButton", mlan_return_msg)) {
      if (ibutton_debug)
        syslog(LOG_AUTHPRIV | LOG_ERR, "%s", mlan_return_msg);
      return(PAM_PERM_DENIED);
    }
/*
    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_DEBUG, "%s", mlan_return_msg);
*/

    /* search our iButton */
    MLanSerialNum(ibutton_rom, FALSE);
    if (!MLanAccess()) {
      if (ibutton_debug)
        syslog(LOG_AUTHPRIV | LOG_ERR, "couldn't find the iButton on the bus");
      Release1WireNet(mlan_return_msg); // release the 1-Wire Net
      if (ibutton_debug)
        syslog(LOG_AUTHPRIV | LOG_DEBUG, "%s", mlan_return_msg);
      return (PAM_PERM_DENIED);
    }

    /* update content */
    if (!MLanWritePacketStd(0, md5_digest, 16, 0, 0)) {
      if (ibutton_debug)
        syslog(LOG_AUTHPRIV | LOG_ERR, "couldn't update the iButton");
      Release1WireNet(mlan_return_msg); // release the 1-Wire Net
      if (ibutton_debug)
        syslog(LOG_AUTHPRIV | LOG_DEBUG, "%s", mlan_return_msg);
      return (PAM_PERM_DENIED);
    }


    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_ERR, "iButton updated");
    Release1WireNet(mlan_return_msg); // release the 1-Wire Net
/*
    if (ibutton_debug)
      syslog(LOG_AUTHPRIV | LOG_DEBUG, "%s", mlan_return_msg);
*/
    return (PAM_SUCCESS);

  }

  if (ibutton_debug)
    syslog(LOG_AUTHPRIV | LOG_ERR, "unknown flags");
  return (PAM_PERM_DENIED);

}

/*------------------------------------------------------------------------------

int iButton_get_local_IP_addr(char *addr, int len)

DESCRIPTION:

- get the local IP address, returning it via the pre-allocated string addr,
  with maximum len size;

- return codes:
  -1 : error
   0 : success

------------------------------------------------------------------------------*/

int iButton_get_local_IP_addr(char *addr, int len)
{
  char                    name[256];
  struct hostent          *he;
  struct in_addr          in;

  if (gethostname(name, sizeof(name))<0)
    return(-1);

  if ((he=gethostbyname(name))==NULL)
    return(-1);

  memcpy(&in.s_addr, *(he->h_addr_list), sizeof(in.s_addr));

  strncpy(addr, inet_ntoa(in), len);

  addr[len-1]='\0';

  return(0);
}
